# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ui_config import UiConfig
from .weather_layer_type import WeatherLayerType


class WeatherLayer(UniversalBaseModel):
    """
    Weather-specific layer with meteorological data visualization.
    """

    id: typing.Optional[str] = None
    user_id: typing_extensions.Annotated[str, FieldMetadata(alias="userId")]
    name: str = pydantic.Field()
    """
    User-defined name for the layer, e.g., 'Danish Wind Turbines'.
    """

    deckgl_props: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="deckglProps")
    ] = pydantic.Field(default=None)
    """
    Base DeckGL layer configuration (styling, interactions, etc.).
    """

    ui_config: typing_extensions.Annotated[typing.Optional[UiConfig], FieldMetadata(alias="uiConfig")] = pydantic.Field(
        default=None
    )
    """
    Configuration for interactive UI controls that can modify layer properties.
    """

    created_at: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="createdAt")] = None
    updated_at: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="updatedAt")] = None
    type: WeatherLayerType = pydantic.Field()
    """
    Weather layer type identifier
    """

    weather_model_id: typing_extensions.Annotated[str, FieldMetadata(alias="weatherModelId")] = pydantic.Field()
    """
    Weather model identifier, e.g., 'ECMF_IFS', 'ERA5', 'NCEP_GFS'
    """

    variable: str = pydantic.Field()
    """
    Weather variable, e.g., 'WindSpeed', 'Temperature', 'CloudCover'
    """

    altitude: int = pydantic.Field()
    """
    Height in meters
    """

    run_hour: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="runHour")] = pydantic.Field(
        default=None
    )
    """
    Forecast run hour (0, 6, 12, 18) - only for forecast models
    """

    run_day: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="runDay")] = pydantic.Field(
        default=None
    )
    """
    Forecast run day in ISO format (YYYY-MM-DD) or 'latest' (case-insensitive) - only for forecast models
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class TimeSeriesDataType(UniversalBaseModel):
    unit: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unit of measurement, e.g., 'MW'
    """

    forecast_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="forecastType")] = (
        pydantic.Field(default=None)
    )
    """
    Type of forecast data, e.g., 'Offshore Wind'
    """

    price_area: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="priceArea")] = pydantic.Field(
        default=None
    )
    """
    Price area identifier, e.g., 'DK1'
    """

    forecast_horizon: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="forecastHorizon")] = (
        pydantic.Field(default=None)
    )
    """
    Forecast horizon, e.g., 'ForecastDayAhead'
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.dataset import Dataset
from ..types.http_validation_error import HttpValidationError
from ..types.time_series_data_point import TimeSeriesDataPoint
from .types.create_geospatial_static_api_v_1_dashboard_datasets_geospatial_static_post_request_geometry_type import (
    CreateGeospatialStaticApiV1DashboardDatasetsGeospatialStaticPostRequestGeometryType,
)
from .types.create_timeseries_live_api_v_1_dashboard_datasets_timeseries_live_post_request_data_format import (
    CreateTimeseriesLiveApiV1DashboardDatasetsTimeseriesLivePostRequestDataFormat,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDatasetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_timeseries_static(
        self,
        *,
        name: str,
        data: typing.Sequence[TimeSeriesDataPoint],
        description: typing.Optional[str] = None,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Dataset]:
        """
        Create a static timeseries dataset.

        Parameters
        ----------
        name : str

        data : typing.Sequence[TimeSeriesDataPoint]

        description : typing.Optional[str]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Dataset]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets/timeseries-static",
            method="POST",
            params={
                "name": name,
                "description": description,
            },
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=typing.Sequence[TimeSeriesDataPoint], direction="write"
                ),
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_timeseries_live(
        self,
        *,
        name: str,
        url: str,
        data_format: typing.Optional[
            CreateTimeseriesLiveApiV1DashboardDatasetsTimeseriesLivePostRequestDataFormat
        ] = None,
        datetime_format: typing.Optional[str] = None,
        description: typing.Optional[str] = None,
        params: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        aliases: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        headers: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Dataset]:
        """
        Create a live timeseries dataset.

        Parameters
        ----------
        name : str

        url : str

        data_format : typing.Optional[CreateTimeseriesLiveApiV1DashboardDatasetsTimeseriesLivePostRequestDataFormat]

        datetime_format : typing.Optional[str]

        description : typing.Optional[str]

        params : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        aliases : typing.Optional[typing.Dict[str, typing.Optional[str]]]

        headers : typing.Optional[typing.Dict[str, typing.Optional[str]]]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Dataset]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets/timeseries-live",
            method="POST",
            params={
                "name": name,
                "url": url,
                "data_format": data_format,
                "datetime_format": datetime_format,
                "description": description,
            },
            json={
                "params": params,
                "aliases": aliases,
                "headers": headers,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_tabular_static(
        self,
        *,
        name: str,
        data: typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]],
        description: typing.Optional[str] = None,
        column_schema: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Optional[typing.Any]]:
        """
        Create a static tabular dataset (placeholder).

        Parameters
        ----------
        name : str

        data : typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]]

        description : typing.Optional[str]

        column_schema : typing.Optional[typing.Dict[str, typing.Optional[str]]]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets/tabular-static",
            method="POST",
            params={
                "name": name,
                "description": description,
            },
            json={
                "data": data,
                "column_schema": column_schema,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_geospatial_static(
        self,
        *,
        name: str,
        data: typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]],
        coordinate_system: typing.Optional[str] = None,
        geometry_type: typing.Optional[
            CreateGeospatialStaticApiV1DashboardDatasetsGeospatialStaticPostRequestGeometryType
        ] = None,
        description: typing.Optional[str] = None,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Dataset]:
        """
        Create a static geospatial dataset compatible with deck.gl data formats.

        Parameters
        ----------
        name : str

        data : typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]]

        coordinate_system : typing.Optional[str]

        geometry_type : typing.Optional[CreateGeospatialStaticApiV1DashboardDatasetsGeospatialStaticPostRequestGeometryType]

        description : typing.Optional[str]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Dataset]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets/geospatial-static",
            method="POST",
            params={
                "name": name,
                "coordinate_system": coordinate_system,
                "geometry_type": geometry_type,
                "description": description,
            },
            json={
                "data": data,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_datasets(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[Dataset]]:
        """
        List all datasets for the user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Dataset]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[Dataset],
                    parse_obj_as(
                        type_=typing.List[Dataset],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_dataset(
        self, dataset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Dataset]:
        """
        Retrieve a single dataset by its ID.

        Parameters
        ----------
        dataset_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Dataset]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/datasets/{jsonable_encoder(dataset_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_dataset(
        self,
        dataset_id: str,
        *,
        name: str,
        description: typing.Optional[str] = None,
        request: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Dataset]:
        """
        Update a dataset's basic properties (name, description, metadata only).

        Parameters
        ----------
        dataset_id : str

        name : str

        description : typing.Optional[str]

        request : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Dataset]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/datasets/{jsonable_encoder(dataset_id)}",
            method="PUT",
            params={
                "name": name,
                "description": description,
            },
            json=request,
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_dataset(
        self, dataset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Delete a dataset.

        Parameters
        ----------
        dataset_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/datasets/{jsonable_encoder(dataset_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_dataset_data(
        self,
        dataset_id: str,
        *,
        start: typing.Optional[str] = None,
        end: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Optional[typing.Any]]:
        """
        Get dataset data with optional time filtering. Works for static and live datasets of all types.

        Parameters
        ----------
        dataset_id : str

        start : typing.Optional[str]

        end : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/datasets/{jsonable_encoder(dataset_id)}/data",
            method="GET",
            params={
                "start": start,
                "end": end,
            },
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDatasetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_timeseries_static(
        self,
        *,
        name: str,
        data: typing.Sequence[TimeSeriesDataPoint],
        description: typing.Optional[str] = None,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Dataset]:
        """
        Create a static timeseries dataset.

        Parameters
        ----------
        name : str

        data : typing.Sequence[TimeSeriesDataPoint]

        description : typing.Optional[str]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Dataset]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets/timeseries-static",
            method="POST",
            params={
                "name": name,
                "description": description,
            },
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=typing.Sequence[TimeSeriesDataPoint], direction="write"
                ),
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_timeseries_live(
        self,
        *,
        name: str,
        url: str,
        data_format: typing.Optional[
            CreateTimeseriesLiveApiV1DashboardDatasetsTimeseriesLivePostRequestDataFormat
        ] = None,
        datetime_format: typing.Optional[str] = None,
        description: typing.Optional[str] = None,
        params: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        aliases: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        headers: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Dataset]:
        """
        Create a live timeseries dataset.

        Parameters
        ----------
        name : str

        url : str

        data_format : typing.Optional[CreateTimeseriesLiveApiV1DashboardDatasetsTimeseriesLivePostRequestDataFormat]

        datetime_format : typing.Optional[str]

        description : typing.Optional[str]

        params : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        aliases : typing.Optional[typing.Dict[str, typing.Optional[str]]]

        headers : typing.Optional[typing.Dict[str, typing.Optional[str]]]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Dataset]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets/timeseries-live",
            method="POST",
            params={
                "name": name,
                "url": url,
                "data_format": data_format,
                "datetime_format": datetime_format,
                "description": description,
            },
            json={
                "params": params,
                "aliases": aliases,
                "headers": headers,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_tabular_static(
        self,
        *,
        name: str,
        data: typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]],
        description: typing.Optional[str] = None,
        column_schema: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Optional[typing.Any]]:
        """
        Create a static tabular dataset (placeholder).

        Parameters
        ----------
        name : str

        data : typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]]

        description : typing.Optional[str]

        column_schema : typing.Optional[typing.Dict[str, typing.Optional[str]]]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets/tabular-static",
            method="POST",
            params={
                "name": name,
                "description": description,
            },
            json={
                "data": data,
                "column_schema": column_schema,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_geospatial_static(
        self,
        *,
        name: str,
        data: typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]],
        coordinate_system: typing.Optional[str] = None,
        geometry_type: typing.Optional[
            CreateGeospatialStaticApiV1DashboardDatasetsGeospatialStaticPostRequestGeometryType
        ] = None,
        description: typing.Optional[str] = None,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Dataset]:
        """
        Create a static geospatial dataset compatible with deck.gl data formats.

        Parameters
        ----------
        name : str

        data : typing.Sequence[typing.Dict[str, typing.Optional[typing.Any]]]

        coordinate_system : typing.Optional[str]

        geometry_type : typing.Optional[CreateGeospatialStaticApiV1DashboardDatasetsGeospatialStaticPostRequestGeometryType]

        description : typing.Optional[str]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Dataset]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets/geospatial-static",
            method="POST",
            params={
                "name": name,
                "coordinate_system": coordinate_system,
                "geometry_type": geometry_type,
                "description": description,
            },
            json={
                "data": data,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_datasets(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[Dataset]]:
        """
        List all datasets for the user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Dataset]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/dashboard/datasets",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[Dataset],
                    parse_obj_as(
                        type_=typing.List[Dataset],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_dataset(
        self, dataset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Dataset]:
        """
        Retrieve a single dataset by its ID.

        Parameters
        ----------
        dataset_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Dataset]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/datasets/{jsonable_encoder(dataset_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_dataset(
        self,
        dataset_id: str,
        *,
        name: str,
        description: typing.Optional[str] = None,
        request: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Dataset]:
        """
        Update a dataset's basic properties (name, description, metadata only).

        Parameters
        ----------
        dataset_id : str

        name : str

        description : typing.Optional[str]

        request : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Dataset]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/datasets/{jsonable_encoder(dataset_id)}",
            method="PUT",
            params={
                "name": name,
                "description": description,
            },
            json=request,
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Dataset,
                    parse_obj_as(
                        type_=Dataset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_dataset(
        self, dataset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete a dataset.

        Parameters
        ----------
        dataset_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/datasets/{jsonable_encoder(dataset_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_dataset_data(
        self,
        dataset_id: str,
        *,
        start: typing.Optional[str] = None,
        end: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Optional[typing.Any]]:
        """
        Get dataset data with optional time filtering. Works for static and live datasets of all types.

        Parameters
        ----------
        dataset_id : str

        start : typing.Optional[str]

        end : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/dashboard/datasets/{jsonable_encoder(dataset_id)}/data",
            method="GET",
            params={
                "start": start,
                "end": end,
            },
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    CreateGeospatialStaticApiV1DashboardDatasetsGeospatialStaticPostRequestGeometryType,
    CreateTimeseriesLiveApiV1DashboardDatasetsTimeseriesLivePostRequestDataFormat,
)

__all__ = [
    "CreateGeospatialStaticApiV1DashboardDatasetsGeospatialStaticPostRequestGeometryType",
    "CreateTimeseriesLiveApiV1DashboardDatasetsTimeseriesLivePostRequestDataFormat",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .list_components_api_v_1_dashboard_components_get_response_item import (
    ListComponentsApiV1DashboardComponentsGetResponseItem,
    ListComponentsApiV1DashboardComponentsGetResponseItem_Map,
    ListComponentsApiV1DashboardComponentsGetResponseItem_Timeseries,
)

__all__ = [
    "ListComponentsApiV1DashboardComponentsGetResponseItem",
    "ListComponentsApiV1DashboardComponentsGetResponseItem_Map",
    "ListComponentsApiV1DashboardComponentsGetResponseItem_Timeseries",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .geometrytype import Geometrytype


class Typeconfig_Geospatial(UniversalBaseModel):
    data_type: typing_extensions.Annotated[typing.Literal["geospatial"], FieldMetadata(alias="dataType")] = "geospatial"
    coordinate_system: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="coordinateSystem")] = None
    geometry_type: typing_extensions.Annotated[Geometrytype, FieldMetadata(alias="geometryType")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Typeconfig_Tabular(UniversalBaseModel):
    data_type: typing_extensions.Annotated[typing.Literal["tabular"], FieldMetadata(alias="dataType")] = "tabular"
    column_schema: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]], FieldMetadata(alias="columnSchema")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Typeconfig_Timeseries(UniversalBaseModel):
    data_type: typing_extensions.Annotated[typing.Literal["timeseries"], FieldMetadata(alias="dataType")] = "timeseries"
    unit: typing.Optional[str] = None
    forecast_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="forecastType")] = None
    price_area: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="priceArea")] = None
    forecast_horizon: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="forecastHorizon")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


Typeconfig = typing.Union[Typeconfig_Geospatial, Typeconfig_Tabular, Typeconfig_Timeseries]



# This file was auto-generated by Fern from our API Definition.

import typing

CreateGeospatialStaticApiV1DashboardDatasetsGeospatialStaticPostRequestGeometryType = typing.Union[
    typing.Literal["point", "line", "polygon"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.map_config import MapConfig
from ...types.time_series_config import TimeSeriesConfig


class ListComponentsApiV1DashboardComponentsGetResponseItem_Map(UniversalBaseModel):
    type: typing.Literal["map"] = "map"
    id: typing.Optional[str] = None
    user_id: typing_extensions.Annotated[str, FieldMetadata(alias="userId")]
    name: str
    config: MapConfig
    layer_ids: typing_extensions.Annotated[typing.Optional[typing.List[str]], FieldMetadata(alias="layerIds")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListComponentsApiV1DashboardComponentsGetResponseItem_Timeseries(UniversalBaseModel):
    type: typing.Literal["timeseries"] = "timeseries"
    id: typing.Optional[str] = None
    user_id: typing_extensions.Annotated[str, FieldMetadata(alias="userId")]
    name: str
    config: TimeSeriesConfig

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ListComponentsApiV1DashboardComponentsGetResponseItem = typing.Union[
    ListComponentsApiV1DashboardComponentsGetResponseItem_Map,
    ListComponentsApiV1DashboardComponentsGetResponseItem_Timeseries,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .layout import Layout
from .react_grid_layout_config import ReactGridLayoutConfig


class Dashboard(UniversalBaseModel):
    id: typing.Optional[str] = None
    name: str
    user_id: typing_extensions.Annotated[str, FieldMetadata(alias="userId")]
    layout: typing.Optional[typing.List[Layout]] = pydantic.Field(default=None)
    """
    Single layout array for all components
    """

    react_grid_layout_config: typing_extensions.Annotated[
        typing.Optional[ReactGridLayoutConfig], FieldMetadata(alias="reactGridLayoutConfig")
    ] = None
    components: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of all component IDs on this dashboard.
    """

    component_state: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="componentState")
    ] = pydantic.Field(default=None)
    """
    Component-specific state like layer ordering, stored as JSON
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.image_meta_response import ImageMetaResponse
from ..types.model_meta import ModelMeta
from ..types.model_meta_response import ModelMetaResponse
from ..types.model_runs_response import ModelRunsResponse
from .raw_client import AsyncRawWeatherClient, RawWeatherClient


class WeatherClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWeatherClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWeatherClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWeatherClient
        """
        return self._raw_client

    def get_model_list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[ModelMeta]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ModelMeta]
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.weather.get_model_list()
        """
        _response = self._raw_client.get_model_list(request_options=request_options)
        return _response.data

    def get_model_meta(
        self, model: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ModelMetaResponse:
        """
        Parameters
        ----------
        model : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelMetaResponse
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.weather.get_model_meta(
            model="model",
        )
        """
        _response = self._raw_client.get_model_meta(model, request_options=request_options)
        return _response.data

    def get_available_model_runs_for_date(
        self, model: str, year: int, month: int, day: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ModelRunsResponse:
        """
        Parameters
        ----------
        model : str

        year : int

        month : int

        day : int

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelRunsResponse
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.weather.get_available_model_runs_for_date(
            model="model",
            year=1,
            month=1,
            day=1,
        )
        """
        _response = self._raw_client.get_available_model_runs_for_date(
            model, year, month, day, request_options=request_options
        )
        return _response.data

    def get_forecast_run(
        self,
        model: str,
        year: int,
        month: int,
        day: int,
        hour: int,
        variable: str,
        height: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ImageMetaResponse:
        """
        Parameters
        ----------
        model : str

        year : int

        month : int

        day : int

        hour : int

        variable : str

        height : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImageMetaResponse
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.weather.get_forecast_run(
            model="model",
            year=1,
            month=1,
            day=1,
            hour=1,
            variable="variable",
            height="height",
        )
        """
        _response = self._raw_client.get_forecast_run(
            model, year, month, day, hour, variable, height, request_options=request_options
        )
        return _response.data

    def get_historical_data(
        self,
        model: str,
        variable: str,
        height: str,
        *,
        start_date: str,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ImageMetaResponse:
        """
        Fetch historical images for a given model/variable/height and date range.
        Uses new storage structure: ERA5/CloudCover/Height_High/YYYY/MM/DD/filename.webp
        This allows fetching all images for a variable/height in a single S3 call.

        Parameters
        ----------
        model : str

        variable : str

        height : str

        start_date : str
            Start date in YYYY-MM-DD format

        end_date : typing.Optional[str]
            End date in YYYY-MM-DD format (optional, defaults to start_date, inclusive)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImageMetaResponse
            Successful Response

        Examples
        --------
        from rebase_dashboard import RebaseDashboardApi

        client = RebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.weather.get_historical_data(
            model="model",
            variable="variable",
            height="height",
            start_date="start_date",
        )
        """
        _response = self._raw_client.get_historical_data(
            model, variable, height, start_date=start_date, end_date=end_date, request_options=request_options
        )
        return _response.data


class AsyncWeatherClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWeatherClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWeatherClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWeatherClient
        """
        return self._raw_client

    async def get_model_list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ModelMeta]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ModelMeta]
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.weather.get_model_list()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_model_list(request_options=request_options)
        return _response.data

    async def get_model_meta(
        self, model: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ModelMetaResponse:
        """
        Parameters
        ----------
        model : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelMetaResponse
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.weather.get_model_meta(
                model="model",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_model_meta(model, request_options=request_options)
        return _response.data

    async def get_available_model_runs_for_date(
        self, model: str, year: int, month: int, day: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ModelRunsResponse:
        """
        Parameters
        ----------
        model : str

        year : int

        month : int

        day : int

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelRunsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.weather.get_available_model_runs_for_date(
                model="model",
                year=1,
                month=1,
                day=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_available_model_runs_for_date(
            model, year, month, day, request_options=request_options
        )
        return _response.data

    async def get_forecast_run(
        self,
        model: str,
        year: int,
        month: int,
        day: int,
        hour: int,
        variable: str,
        height: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ImageMetaResponse:
        """
        Parameters
        ----------
        model : str

        year : int

        month : int

        day : int

        hour : int

        variable : str

        height : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImageMetaResponse
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.weather.get_forecast_run(
                model="model",
                year=1,
                month=1,
                day=1,
                hour=1,
                variable="variable",
                height="height",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_forecast_run(
            model, year, month, day, hour, variable, height, request_options=request_options
        )
        return _response.data

    async def get_historical_data(
        self,
        model: str,
        variable: str,
        height: str,
        *,
        start_date: str,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ImageMetaResponse:
        """
        Fetch historical images for a given model/variable/height and date range.
        Uses new storage structure: ERA5/CloudCover/Height_High/YYYY/MM/DD/filename.webp
        This allows fetching all images for a variable/height in a single S3 call.

        Parameters
        ----------
        model : str

        variable : str

        height : str

        start_date : str
            Start date in YYYY-MM-DD format

        end_date : typing.Optional[str]
            End date in YYYY-MM-DD format (optional, defaults to start_date, inclusive)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImageMetaResponse
            Successful Response

        Examples
        --------
        import asyncio

        from rebase_dashboard import AsyncRebaseDashboardApi

        client = AsyncRebaseDashboardApi(
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.weather.get_historical_data(
                model="model",
                variable="variable",
                height="height",
                start_date="start_date",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_historical_data(
            model, variable, height, start_date=start_date, end_date=end_date, request_options=request_options
        )
        return _response.data
